@@ -35,11 +35,14 @@ from . import input_event
 from . import messages
 from . import orca_state
 from . import script_manager
+from . import settings
 
 _scriptManager = script_manager.getManager()
 
 class EventManager:
 
+    EMBEDDED_OBJECT_CHARACTER = '\ufffc'
+
     def __init__(self, asyncMode=True):
         debug.println(debug.LEVEL_FINEST, 'INFO: Initializing event manager')
         debug.println(debug.LEVEL_FINEST, 'INFO: Async Mode is %s' % asyncMode)
@@ -99,12 +102,89 @@ class EventManager:
         if list(filter(event.type.startswith, self._ignoredEvents)):
             return True
 
+        if event.type.startswith('window'):
+            return False
+
         # This should ultimately be changed as there are valid reasons
         # to handle these events at the application level.
         if event.type.startswith('object:children-changed:remove') \
            and event.source != self.registry.getDesktop(0):
             return True
 
+        if event.type.startswith('object:text-changed') and event.type.endswith('system'):
+            # We should also get children-changed events telling us the same thing.
+            # Getting a bunch of both can result in a flood that grinds us to a halt.
+            if event.any_data == self.EMBEDDED_OBJECT_CHARACTER:
+                msg = 'INFO: Text changed event for embedded object. Who cares?'
+                debug.println(debug.LEVEL_INFO, msg)
+                return True
+
+        if settings.disableDeadAccessibleFilter:
+            return False
+
+        try:
+            # TODO - JD: For now we won't ask for the name. Simply asking for the name should
+            # not break anything, and should be a reliable way to quickly identify defunct
+            # objects. But apparently the mere act of asking for the name causes Orca to stop
+            # presenting Eclipse (and possibly other) applications. This might be an AT-SPI2
+            # issue, but until we know for certain....
+            #name = event.source.name
+            state = event.source.getState()
+        except:
+            msg = 'ERROR: %s from potentially-defunct source %s in app %s (%s, %s, %s)' % \
+                  (event.type, event.source, event.host_application, event.detail1,
+                   event.detail2, event.any_data)
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+        if state.contains(pyatspi.STATE_DEFUNCT):
+            msg = 'ERROR: %s from defunct source %s in app %s (%s, %s, %s)' % \
+                  (event.type, event.source, event.host_application, event.detail1,
+                   event.detail2, event.any_data)
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
+        if event.type.startswith('object:state-changed:showing'):
+            try:
+                role = event.source.getRole()
+            except:
+                role = None
+            if role == pyatspi.ROLE_MENU_ITEM:
+                msg = 'INFO: %s for %s in app %s. Who cares?' % \
+                      (event.type, event.source, event.host_application)
+                debug.println(debug.LEVEL_INFO, msg)
+                return True
+
+        if event.type.startswith('object:children-changed:add'):
+            if not event.any_data:
+                msg = 'ERROR: %s without child from source %s in app %s' % \
+                      (event.type, event.source, event.host_application)
+                debug.println(debug.LEVEL_INFO, msg)
+                return True
+            try:
+                state = event.any_data.getState()
+                role = event.any_data.getRole()
+            except:
+                msg = 'ERROR: %s with potentially-defunct child %s from source %s in app %s' % \
+                      (event.type, event.any_data, event.source, event.host_application)
+                debug.println(debug.LEVEL_INFO, msg)
+                return True
+            if state.contains(pyatspi.STATE_DEFUNCT):
+                msg = 'ERROR: %s with defunct child %s from source %s in app %s' % \
+                      (event.type, event.any_data, event.source, event.host_application)
+                debug.println(debug.LEVEL_INFO, msg)
+                return True
+
+            # This should be safe. We do not have a reason to present a newly-added,
+            # but not focused image. We do not need to update live regions for images.
+            # This is very likely a completely and utterly useless event for us. The
+            # reason for ignoring it here rather than quickly processing it is the
+            # potential for event floods like we're seeing from matrix.org.
+            if role == pyatspi.ROLE_IMAGE:
+                msg = 'INFO: %s for child image %s from source %s in app %s. Who cares?' % \
+                      (event.type, event.any_data, event.source, event.host_application)
+                debug.println(debug.LEVEL_INFO, msg)
+                return True
+
         return False
 
     def _addToQueue(self, event, asyncMode):
@@ -142,15 +222,15 @@ class EventManager:
         elif isinstance(e, input_event.BrailleEvent):
             data = "'%s'" % repr(e.event)
         elif not debug.eventDebugFilter or debug.eventDebugFilter.match(e.type):
-            data = ""
+            data = "%s (%s,%s,%s)" % (e.source, e.detail1, e.detail2, e.any_data)
         else:
             return
 
         eType = str(e.type).upper()
         if isEnqueue:
-            string = "----------> QUEUEING %s %s" % (eType, data)
+            string = "----------> QUEUEING %s %s" % (eType, data.upper())
         else:
-            string = "DEQUEUED %s %s <----------" % (eType, data)
+            string = "DEQUEUED %s %s <----------" % (eType, data.upper())
 
         debug.println(debug.LEVEL_ALL, string)
 
@@ -554,10 +634,24 @@ class EventManager:
         debug.println(debug.LEVEL_FINEST, "Script for event: %s" % script.name)
         setNewActiveScript, reason = self._isActivatableEvent(event, script)
         if setNewActiveScript:
-            app = event.host_application or event.source.getApplication()
-            _scriptManager.setActiveScript(script, reason)
+            try:
+                app = event.host_application or event.source.getApplication()
+            except:
+                msg = 'ERROR: Could not get application for %s' % event.source
+                debug.println(debug.LEVEL_INFO, msg)
+                return
+            try:
+                _scriptManager.setActiveScript(script, reason)
+            except:
+                msg = 'ERROR: Could not set active script for %s' % event.source
+                debug.println(debug.LEVEL_INFO, msg)
+                return
 
-        script.processObjectEvent(event)
+        try:
+            script.processObjectEvent(event)
+        except:
+            msg = 'ERROR: Could not process %s' % event.type
+            debug.println(debug.LEVEL_INFO, msg)
 
     def processKeyboardEvent(self, keyboardEvent):
         """Processes the given keyboard event based on the keybinding from the
