@@ -96,15 +96,15 @@ formatting = {
             },
         'suffix': {
             'focused': '[]',
-            'unfocused': 'newNodeLevel + unselectedCell + ' + TUTORIAL,
-            'basicWhereAmI': TUTORIAL + ' + description',
+            'unfocused': 'newNodeLevel + unselectedCell + clickable + hasLongDesc + ' + TUTORIAL + ' + description',
+            'basicWhereAmI': TUTORIAL + ' + clickable + hasLongDesc + description',
             'detailedWhereAmI' : '[]'
             },
         'default': {
             'focused': '[]',
             'unfocused': 'labelOrName + allTextSelection + roleName + availability + ' + MNEMONIC + ' + accelerator + childWidget',
             'basicWhereAmI': 'labelOrName + roleName',
-            'detailedWhereAmI' : '[]'
+            'detailedWhereAmI' : 'pageSummary'
             },
         pyatspi.ROLE_ALERT: {
             'unfocused': 'expandedEOCs or (labelOrName + unrelatedLabels)'
@@ -118,6 +118,9 @@ formatting = {
             'basicWhereAmI': 'parentRoleName + labelAndName + selectedItemCount',
             'detailedWhereAmI': 'parentRoleName + labelAndName + selectedItemCount + selectedItems'
             },
+        pyatspi.ROLE_CAPTION: {
+            'unfocused': '((substring and currentLineText) or labelAndName) + roleName'
+            },
         pyatspi.ROLE_CHECK_BOX: {
             'focused': 'checkedState',
             'unfocused': 'labelOrName + roleName + checkedState + required + availability + ' + MNEMONIC + ' + accelerator',
@@ -133,6 +136,9 @@ formatting = {
             'unfocused': 'label + roleName + value + required + availability + ' + MNEMONIC,
             'basicWhereAmI': 'label + roleName + value + percentage + ' + MNEMONIC + ' + accelerator + required'
             },
+        pyatspi.ROLE_COLUMN_HEADER: {
+            'unfocused': '((substring and currentLineText) or labelAndName) + roleName'
+            },
         pyatspi.ROLE_COMBO_BOX: {
             'focused': 'expandableState',
             'unfocused': 'label + name + roleName + positionInList + ' + MNEMONIC + ' + accelerator',
@@ -152,12 +158,19 @@ formatting = {
             'basicWhereAmI': 'label + readOnly + textRole + textContent + anyTextSelection + ' + MNEMONIC,
             'detailedWhereAmI': 'label + readOnly + textRole + textContentWithAttributes + anyTextSelection + ' + MNEMONIC + ' + ' + TUTORIAL
             },
+        pyatspi.ROLE_EMBEDDED: {
+            'focused': 'labelOrName + roleName',
+            'unfocused': '(expandedEOCs or (labelOrName + unrelatedLabels)) + roleName'
+            },
         pyatspi.ROLE_ENTRY: {
             'focused': 'labelOrName + readOnly + textRole + (currentLineText or placeholderText) + allTextSelection',
             'unfocused': 'labelOrName + readOnly + textRole + (currentLineText or placeholderText) + allTextSelection + ' + MNEMONIC,
             'basicWhereAmI': 'labelOrName + readOnly + textRole + (textContent or placeholderText) + anyTextSelection + ' + MNEMONIC,
             'detailedWhereAmI': 'labelOrName + readOnly + textRole + (textContentWithAttributes or placeholderText) + anyTextSelection + ' + MNEMONIC + ' + ' + TUTORIAL
             },
+        pyatspi.ROLE_FOOTER: {
+            'unfocused': '(displayedText or name) + roleName',
+        },
         pyatspi.ROLE_FORM: {
             'focused': 'labelOrName + readOnly + textRole + currentLineText + allTextSelection',
             'unfocused': 'labelOrName + readOnly + textRole + currentLineText + allTextSelection + ' + MNEMONIC,
@@ -167,7 +180,8 @@ formatting = {
             'unfocused': 'labelOrName + allTextSelection + roleName + unfocusedDialogCount + availability'
             },
         pyatspi.ROLE_HEADING: {
-            'unfocused': 'displayedText + roleName + expandableState + availability + ' + MNEMONIC,
+            'focused': 'displayedText + roleName + expandableState',
+            'unfocused': 'displayedText + roleName + expandableState',
             'basicWhereAmI': 'label + readOnly + textRole + textContent + anyTextSelection + ' + MNEMONIC,
             'detailedWhereAmI': 'label + readOnly + textRole + textContentWithAttributes + anyTextSelection + ' + MNEMONIC + ' + ' + TUTORIAL
             },
@@ -177,6 +191,9 @@ formatting = {
             'basicWhereAmI': 'parentRoleName + labelAndName + selectedItemCount',
             'detailedWhereAmI': 'parentRoleName + labelAndName + selectedItemCount + selectedItems'
             },
+        pyatspi.ROLE_IMAGE: {
+            'unfocused': 'labelAndName + roleName'
+            },
         pyatspi.ROLE_INFO_BAR: {
             'unfocused': 'labelAndName + unrelatedLabels'
             },
@@ -274,6 +291,9 @@ formatting = {
             'unfocused': 'labelOrName + radioState + roleName + availability + ' + MNEMONIC + ' + accelerator + positionInList',
             'basicWhereAmI': 'ancestors + labelOrName + roleName + radioState + accelerator + positionInList + ' + MNEMONIC
             },
+        pyatspi.ROLE_ROW_HEADER: {
+            'unfocused': '((substring and currentLineText) or labelAndName) + roleName'
+            },
         pyatspi.ROLE_SCROLL_BAR: {
             'focused': 'value',
             'unfocused': 'labelOrName + roleName + value + required + availability + ' + MNEMONIC,
@@ -298,6 +318,12 @@ formatting = {
             'unfocused': 'labelAndName + roleName + value + availability + ' + MNEMONIC,
             'basicWhereAmI' : 'labelAndName + roleName + value'
             },
+        # TODO - JD: There is now an actual ROLE_STATIC in ATK and AT-SPI2. Next
+        # time we need to bump dependencies for more significant things, we need
+        # to remove this fake role and use it instead.
+        'ROLE_STATIC': {
+            'unfocused': '(displayedText or name) + roleName',
+        },
         pyatspi.ROLE_TABLE: {
             'focused': 'labelAndName + table',
             'unfocused': 'labelAndName + table',
@@ -613,7 +639,9 @@ formatting = {
                                      asString((labelOrName or description) + expandableState + roleName),\
                                      indicator=asString(toggleState))]'
             },
-        #pyatspi.ROLE_TOOL_BAR: 'default'
+        pyatspi.ROLE_TOOL_BAR: {
+            'unfocused': '[Component(obj, asString(labelOrName + roleName))]',
+            },
         #pyatspi.ROLE_TREE: 'default'
         #pyatspi.ROLE_TREE_TABLE: 'default'
         #pyatspi.ROLE_WINDOW: 'default'
@@ -635,8 +663,8 @@ if settings.useExperimentalSpeechProsody:
     formatting['speech'][pyatspi.ROLE_COMBO_BOX]['unfocused'] = 'label + name + roleName + pause + positionInList + ' + MNEMONIC + ' + accelerator'
     formatting['speech'][pyatspi.ROLE_COMBO_BOX]['basicWhereAmI'] = \
         'label + roleName + pause + name + positionInList + ' + MNEMONIC + ' + accelerator'
-    formatting['speech'][pyatspi.ROLE_HEADING]['unfocused'] = \
-        'displayedText + roleName + expandableState + availability + ' + MNEMONIC
+    formatting['speech'][pyatspi.ROLE_HEADING]['focused'] = 'displayedText + roleName + expandableState'
+    formatting['speech'][pyatspi.ROLE_HEADING]['unfocused'] = 'displayedText + roleName + expandableState'
     formatting['speech'][pyatspi.ROLE_HEADING]['basicWhereAmI'] = \
         'label + readOnly + textRole + pause + textContent + anyTextSelection + ' + MNEMONIC
     formatting['speech'][pyatspi.ROLE_HEADING]['detailedWhereAmI'] = \
