@@ -164,11 +164,12 @@ CAPITALIZATION_SPELL_BRIEF = C_("capitalization style", "spell")
 # to get into a GUI.
 CAPITALIZATION_SPELL_FULL = _("Capitalization style set to spell.")
 
-# Translators: Gecko native caret navigation is where Firefox (or Thunderbird)
-# itself controls how the arrow keys move the caret around HTML content. It's
-# often broken, so Orca needs to provide its own support. As such, Orca offers
-# the user the ability to toggle which application is controlling the caret.
-CARET_CONTROL_GECKO = _("Gecko is controlling the caret.")
+# Translators: Native application caret navigation does not always work as the
+# Orca user wants. As such, Orca offers the user the ability to toggle between
+# the application controlling the caret and Orca controlling it. This message
+# is presented to indicate that the application's native caret navigation is
+# active / not being overridden by Orca.
+CARET_CONTROL_APP = _("The application is controlling the caret.")
 
 # Translators: Gecko native caret navigation is where Firefox (or Thunderbird)
 # itself controls how the arrow keys move the caret around HTML content. It's
@@ -272,6 +273,10 @@ CLI_DEBUG_FILE_NAME = _("FILE")
 # Orca will need to be run from a terminal window.
 CLI_SETUP = _("Set up user preferences (text version)")
 
+# Translators: This is the description of command line option '-s, --setup'
+# that will place the user in Orca's GUI preferences dialog.
+CLI_GUI_SETUP = _("Set up user preferences (GUI version)")
+
 # Translators: This text is the description displayed when Orca is launched
 # from the command line and the help text is displayed.
 CLI_EPILOG = _("Report bugs to orca-list@gnome.org.")
@@ -1105,6 +1110,25 @@ MODE_BROWSE = _("Browse mode")
 # This string is the message presented when Orca switches to focus mode.
 MODE_FOCUS = _("Focus mode")
 
+# Translators: Orca has a number of commands that override the default
+# behavior within an application. For instance, on a web page, "h" moves
+# you to the next heading. What should happen when you press an "h" in
+# an entry on a web page depends: If you want to resume reading content,
+# "h" should move to the next heading; if you want to enter text, "h"
+# should not not move you to the next heading. Similarly, if you are
+# at the bottom of an entry and press Down arrow, should you leave the
+# entry? Again, it depends on if you want to resume reading content or
+# if you are editing the text in the entry. Because Orca doesn't know
+# what you want to do, it has two modes: In browse mode, Orca treats
+# key presses as commands to read the content; in focus mode, Orca treats
+# key presses as something that should be handled by the focused widget.
+# This string is a tutorial message presented to the user who has just
+# navigated to a widget in browse mode to inform them of the keystroke
+# they must press to enable focus mode for the purposes of interacting
+# with the widget. The substituted string is a human-consumable keybinding
+# such as "Alt+Shift+A."
+MODE_FOCUS_TUTORIAL = _("To enable focus mode press %s.")
+
 # Translators: (Please see the previous, detailed translator notes about
 # Focus mode and Browse mode.) In order to minimize the amount of work Orca
 # users need to do to switch between focus mode and browse mode, Orca attempts
@@ -1476,6 +1500,26 @@ SPEECH_HIGHER = _("higher.")
 # Translators: This string announces speech pitch change.
 SPEECH_LOWER  = _("lower.")
 
+# Translators: This string announces speech volume change.
+SPEECH_LOUDER = _("louder.")
+
+# Translators: This string announces speech volume change.
+SPEECH_SOFTER  = _("softer.")
+
+# Translators: Orca's verbosity levels control how much (or how little)
+# Orca will speak when presenting objects as the user navigates within
+# applications and reads content. The two levels are "brief" and "verbose".
+# The following string is a message spoken to the user upon toggling
+# this setting via command.
+SPEECH_VERBOSITY_BRIEF = C_("Speech", "Verbosity level: brief")
+
+# Translators: Orca's verbosity levels control how much (or how little)
+# Orca will speak when presenting objects as the user navigates within
+# applications and reads content. The two levels are "brief" and "verbose".
+# The following string is a message spoken to the user upon toggling
+# this setting via command.
+SPEECH_VERBOSITY_VERBOSE = C_("Speech", "Verbosity level: verbose")
+
 # Translators: We replace the ellipses (both manual and UTF-8) with a spoken
 # string. The extra space you see at the beginning is because we need the
 # speech synthesis engine to speak the new string well. For example, "Open..."
