@@ -100,6 +100,10 @@ class Script:
         self.utilities = self.getUtilities()
         self.labelInference = self.getLabelInference()
         self.structuralNavigation = self.getStructuralNavigation()
+        self.caretNavigation = self.getCaretNavigation()
+        self.bookmarks = self.getBookmarks()
+        self.liveRegionManager = self.getLiveRegionManager()
+
         self.chat = self.getChat()
         self.inputEventHandlers = {}
         self.pointOfReference = {}
@@ -113,7 +117,6 @@ class Script:
         self.generatorCache = {}
         self.eventCache = {}
         self.whereAmI = self.getWhereAmI()
-        self.bookmarks = self.getBookmarks()
         self.spellcheck = self.getSpellCheck()
         self.voices = settings.voices
         self.tutorialGenerator = self.getTutorialGenerator()
@@ -121,6 +124,7 @@ class Script:
         self.flatReviewContextClass = flat_review.Context
 
         self.findCommandRun = False
+        self._lastCommandWasStructNav = False
 
         debug.println(debug.LEVEL_FINE, "NEW SCRIPT: %s" % self.name)
 
@@ -223,6 +227,10 @@ class Script:
         """Returns the spellcheck support for this script."""
         return None
 
+    def getCaretNavigation(self):
+        """Returns the caret navigation support for this script."""
+        return None
+
     def getUtilities(self):
         """Returns the utilites for this script.
         """
@@ -239,10 +247,14 @@ class Script:
         return []
 
     def getStructuralNavigation(self):
-        """Returns the 'structural navigation' class for this script.
-        """
+        """Returns the 'structural navigation' class for this script."""
         types = self.getEnabledStructuralNavigationTypes()
-        return structural_navigation.StructuralNavigation(self, types)
+        enable = _settingsManager.getSetting('structuralNavigationEnabled')
+        return structural_navigation.StructuralNavigation(self, types, enable)
+
+    def getLiveRegionManager(self):
+        """Returns the live region support for this script."""
+        return None
 
     def useStructuralNavigationModel(self):
         """Returns True if we should use structural navigation. Most
@@ -409,12 +421,17 @@ class Script:
              and event.detail1 == cachedEvent.detail1:
             skip = True
             reason = "more recent event of the same type in a different object"
+        elif event.type.endswith("system") and event.source == cachedEvent.source:
+            skip = True
+            reason = "more recent system event in the same object"
 
         if skip:
             debug.println(debug.LEVEL_FINE,
-                          "script.skipObjectEvent: skipped due to %s" \
+                          "script.skipObjectEvent: skipped due to %s:" \
                           % reason)
-
+            debug.println(debug.LEVEL_FINE,
+                          "\tType: %s\n\tSource: %s\n\tDetail1: %s"
+                          % (cachedEvent.type, cachedEvent.source, cachedEvent.detail1))
         return skip
 
     def checkKeyboardEventData(self, keyboardEvent):
@@ -455,18 +472,23 @@ class Script:
             user_bindings = user_bindings_map["default"]
 
         consumes = False
+        self._lastCommandWasStructNav = False
         if user_bindings:
             handler = user_bindings.getInputHandler(keyboardEvent)
             if handler \
                  and handler.function in self.structuralNavigation.functions:
-                return self.useStructuralNavigationModel()
+                consumes = self.useStructuralNavigationModel()
+                if consumes:
+                    self._lastCommandWasStructNav = True
             else:
                 consumes = handler != None
         if not consumes:
             handler = self.keyBindings.getInputHandler(keyboardEvent)
             if handler \
                  and handler.function in self.structuralNavigation.functions:
-                return self.useStructuralNavigationModel()
+                consumes = self.useStructuralNavigationModel()
+                if consumes:
+                    self._lastCommandWasStructNav = True
             else:
                 consumes = handler != None
         return consumes
