@@ -41,6 +41,7 @@ from . import input_event
 from . import mathsymbols
 from . import messages
 from . import mouse_review
+from . import orca
 from . import orca_state
 from . import object_properties
 from . import pronunciation_dict
@@ -160,6 +161,15 @@ class Utilities:
         -obj: the table cell whose index we need.
         """
 
+        try:
+            attrs = dict([attr.split(':', 1) for attr in obj.getAttributes()])
+        except:
+            attrs = {}
+
+        index = attrs.get('table-cell-index')
+        if index:
+            return int(index)
+
         return obj.getIndexInParent()
 
     def childNodes(self, obj):
@@ -437,11 +447,25 @@ class Utilities:
         self._script.generatorCache[self.DISPLAYED_TEXT][obj] = displayedText
         return self._script.generatorCache[self.DISPLAYED_TEXT][obj]
 
-    def documentFrame(self):
+    def documentFrame(self, obj=None):
         """Returns the document frame which is displaying the content.
         Note that this is intended primarily for web content."""
 
-        return None
+        if not obj:
+            obj, offset = self.getCaretContext()
+        docRoles = [pyatspi.ROLE_DOCUMENT_EMAIL,
+                    pyatspi.ROLE_DOCUMENT_FRAME,
+                    pyatspi.ROLE_DOCUMENT_PRESENTATION,
+                    pyatspi.ROLE_DOCUMENT_SPREADSHEET,
+                    pyatspi.ROLE_DOCUMENT_TEXT,
+                    pyatspi.ROLE_DOCUMENT_WEB]
+        stopRoles = [pyatspi.ROLE_FRAME, pyatspi.ROLE_SCROLL_PANE]
+        document = self.ancestorWithRole(obj, docRoles, stopRoles)
+        if not document and orca_state.locusOfFocus:
+            if orca_state.locusOfFocus.getRole() in docRoles:
+                return orca_state.locusOfFocus
+
+        return document
 
     def documentFrameURI(self):
         """Returns the URI of the document frame that is active."""
@@ -582,6 +606,17 @@ class Utilities:
     def isHidden(self, obj):
         return False
 
+    def isStatic(self, obj):
+        role = obj.getRole()
+        try:
+            isStatic = role == pyatspi.ROLE_STATIC
+        except:
+            isStatic = False
+
+        if not isStatic and role == pyatspi.ROLE_TEXT:
+            isStatic = not obj.getState().contains(pyatspi.STATE_FOCUSABLE)
+        return isStatic
+
     def isLayoutOnly(self, obj):
         """Returns True if the given object is a container which has
         no presentable information (label, name, displayed text, etc.)."""
@@ -1301,6 +1336,26 @@ class Utilities:
 
         return abs(center1 - center2) <= delta
 
+    @staticmethod
+    def pathComparison(path1, path2, treatDescendantAsSame=False):
+        """Compares the two paths and returns -1, 0, or 1 to indicate if path1
+        is before, the same, or after path2."""
+
+        if path1 == path2:
+            return 0
+
+        for x in range(min(len(path1), len(path2))):
+            if path1[x] < path2[x]:
+                return -1
+            if path1[x] > path2[x]:
+                return 1
+
+        if treatDescendantAsSame:
+            return 0
+
+        rv = len(path1) - len(path2)
+        return min(max(rv, -1), 1)
+
     @staticmethod
     def spatialComparison(obj1, obj2):
         """Compares the physical locations of obj1 and obj2 and returns -1,
@@ -1544,8 +1599,7 @@ class Utilities:
 
         return rv
 
-    @staticmethod
-    def characterOffsetInParent(obj):
+    def characterOffsetInParent(self, obj):
         """Returns the character offset of the embedded object
         character for this object in its parent's accessible text.
 
@@ -1652,10 +1706,17 @@ class Utilities:
         Returns True if the word is flagged as misspelled.
         """
 
+        attributes, start, end  = self.textAttributes(obj, offset, True)
+        if attributes.get("invalid") == "spelling":
+            return True
+        if attributes.get("text-spelling") == "misspelled":
+            return True
+        if attributes.get("underline") == "error":
+            return True
+
         return False
 
-    @staticmethod
-    def queryNonEmptyText(obj):
+    def queryNonEmptyText(self, obj):
         """Get the text interface associated with an object, if it is
         non-empty.
 
@@ -1705,6 +1766,21 @@ class Utilities:
 
         return [textContents, startOffset, endOffset]
 
+    def getCaretContext(self):
+        obj = orca_state.locusOfFocus
+        try:
+            offset = obj.queryText().caretOffset
+        except NotImplementedError:
+            offset = 0
+        except:
+            offset = -1
+
+        return obj, offset
+
+    def setCaretPosition(self, obj, offset):
+        orca.setLocusOfFocus(None, obj, False)
+        self.setCaretOffset(obj, offset)
+
     def setCaretOffset(self, obj, offset):
         """Set the caret offset on a given accessible. Similar to
         Accessible.setCaretOffset()
@@ -1769,19 +1845,6 @@ class Utilities:
 
         return rv, start, end
 
-    def unicodeText(self, obj):
-        """Returns the unicode text for an object or None if the object
-        doesn't implement the accessible text specialization.
-        """
-
-        # TODO: eliminate calls to this now-redundant method
-
-        text = self.queryNonEmptyText(obj)
-        if text:
-            return text.getText(0, -1)
-
-        return None
-
     def willEchoCharacter(self, event):
         """Given a keyboard event containing an alphanumeric key,
         determine if the script is likely to echo it as a character.
@@ -2362,23 +2425,10 @@ class Utilities:
             debug.printException(debug.LEVEL_WARNING)
             return ""
 
-    def getLineContentsAtOffset(self, obj, offset, layoutMode=True):
+    def getLineContentsAtOffset(self, obj, offset, layoutMode=True, useCache=True):
         return []
 
-    def getObjectsFromEOCs(self, obj, offset=None, boundary=None):
-        """Breaks the string containing a mixture of text and embedded object
-        characters into a list of (obj, startOffset, endOffset, string) tuples.
-
-        Arguments
-        - obj: the object whose EOCs we need to expand into tuples
-        - offset: the character offset. If None, use the current offset.
-        - boundary: the pyatspi text boundary type. If None, get all text.
-
-        Returns a list of (obj, startOffset, endOffset, string) tuples.
-        """
-
-        # For now, each script should implement this functionality itself.
-
+    def getObjectContentsAtOffset(self, obj, offset=0, useCache=True):
         return []
 
     @staticmethod
@@ -2452,21 +2502,14 @@ class Utilities:
         return False
 
     def headingLevel(self, obj):
-        level = 0
+        if not (obj and obj.getRole() == pyatspi.ROLE_HEADING):
+            return 0
 
-        if obj is None:
-            return level
-
-        if obj.getRole() == pyatspi.ROLE_HEADING:
-            attributes = obj.getAttributes()
-            if attributes is None:
-                return level
-            for attribute in attributes:
-                if attribute.startswith("level:"):
-                    level = int(attribute.split(":")[1])
-                    break
-
-        return level
+        try:
+            attrs = dict([attr.split(':', 1) for attr in obj.getAttributes()])
+        except:
+            return 0
+        return int(attrs.get('level', '0'))
 
     def hasMeaningfulToggleAction(self, obj):
         try:
@@ -2481,6 +2524,48 @@ class Utilities:
 
         return False
 
+    def columnHeadersForCell(self, obj):
+        if not (obj and obj.getRole() == pyatspi.ROLE_TABLE_CELL):
+            return []
+
+        isTable = lambda x: x and 'Table' in pyatspi.listInterfaces(x)
+        parent = pyatspi.findAncestor(obj, isTable)
+        try:
+            table = parent.queryTable()
+        except:
+            return []
+
+        index = self.cellIndex(obj)
+        row, col = table.getRowAtIndex(index), table.getColumnAtIndex(index)
+        colspan = table.getColumnExtentAt(row, col)
+
+        headers = []
+        for c in range(col, col+colspan):
+            headers.append(table.getColumnHeader(c))
+
+        return headers
+
+    def rowHeadersForCell(self, obj):
+        if not (obj and obj.getRole() == pyatspi.ROLE_TABLE_CELL):
+            return []
+
+        isTable = lambda x: x and 'Table' in pyatspi.listInterfaces(x)
+        parent = pyatspi.findAncestor(obj, isTable)
+        try:
+            table = parent.queryTable()
+        except:
+            return []
+
+        index = self.cellIndex(obj)
+        row, col = table.getRowAtIndex(index), table.getColumnAtIndex(index)
+        rowspan = table.getRowExtentAt(row, col)
+
+        headers = []
+        for r in range(row, row+rowspan):
+            headers.append(table.getRowHeader(r))
+
+        return headers
+
     def columnHeaderForCell(self, obj):
         if not (obj and obj.getRole() == pyatspi.ROLE_TABLE_CELL):
             return None
@@ -2512,6 +2597,23 @@ class Utilities:
         return table.getRowHeader(rowIndex)
 
     def coordinatesForCell(self, obj):
+        roles = [pyatspi.ROLE_TABLE_CELL,
+                 pyatspi.ROLE_COLUMN_HEADER,
+                 pyatspi.ROLE_ROW_HEADER]
+        if not (obj and obj.getRole() in roles):
+            return -1, -1
+
+        isTable = lambda x: x and 'Table' in pyatspi.listInterfaces(x)
+        parent = pyatspi.findAncestor(obj, isTable)
+        try:
+            table = parent.queryTable()
+        except:
+            return -1, -1
+
+        index = self.cellIndex(obj)
+        return table.getRowAtIndex(index), table.getColumnAtIndex(index)
+
+    def rowAndColumnSpan(self, obj):
         if not (obj and obj.getRole() == pyatspi.ROLE_TABLE_CELL):
             return -1, -1
 
@@ -2523,7 +2625,30 @@ class Utilities:
             return -1, -1
 
         index = self.cellIndex(obj)
-        return table.getRowAtIndex(index), table.getColumnHeader(index)
+        row, col = table.getRowAtIndex(index), table.getColumnAtIndex(index)
+        return table.getRowExtentAt(row, col), table.getColumnExtentAt(row, col)
+
+    def cellForCoordinates(self, obj, row, column):
+        try:
+            table = obj.queryTable()
+        except:
+            return None
+
+        return table.getAccessibleAt(row, column)
+
+    def isNonUniformTable(self, obj):
+        try:
+            table = obj.queryTable()
+        except:
+            return False
+
+        for r in range(table.nRows):
+            for c in range(table.nColumns):
+                if table.getRowExtentAt(r, c) > 1 \
+                   or table.getColumnExtentAt(r, c) > 1:
+                    return True
+
+        return False
 
     def isZombie(self, obj):
         try:
@@ -2599,16 +2724,19 @@ class Utilities:
         if not obj:
             return -1, -1
 
-        if obj.getRole() == pyatspi.ROLE_COMBO_BOX:
+        isComboBox = obj.getRole() == pyatspi.ROLE_COMBO_BOX
+        if isComboBox:
             selected = self.selectedChildren(obj)
             if selected:
                 obj = selected[0]
 
         parent = self.getFunctionalParent(obj)
         siblings = self.getFunctionalChildren(parent)
-        layoutRoles = [pyatspi.ROLE_SEPARATOR, pyatspi.ROLE_TEAROFF_MENU_ITEM]
-        isNotLayoutOnly = lambda x: not (self.isZombie(x) or x.getRole() in layoutRoles)
-        siblings = list(filter(isNotLayoutOnly, siblings))
+        if not (isComboBox or pyatspi.utils.findAncestor(obj, isComboBox)) \
+           and len(siblings) < 100:
+            layoutRoles = [pyatspi.ROLE_SEPARATOR, pyatspi.ROLE_TEAROFF_MENU_ITEM]
+            isNotLayoutOnly = lambda x: not (self.isZombie(x) or x.getRole() in layoutRoles)
+            siblings = list(filter(isNotLayoutOnly, siblings))
         if not (siblings and obj in siblings):
             return -1, -1
 
