@@ -254,6 +254,20 @@ class Utilities(script_utilities.Utilities):
 
         return [startIndex, endIndex]
 
+    def rowHeadersForCell(self, obj):
+        rowHeader, colHeader = self.getDynamicHeadersForCell(obj)
+        if rowHeader:
+            return [rowHeader]
+
+        return super().rowHeadersForCell(obj)
+
+    def columnHeadersForCell(self, obj):
+        rowHeader, colHeader = self.getDynamicHeadersForCell(obj)
+        if colHeader:
+            return [colHeader]
+
+        return super().columnHeadersForCell(obj)
+
     def getDynamicHeadersForCell(self, obj, onlyIfNew=False):
         if not (self._script.dynamicRowHeaders or self._script.dynamicColumnHeaders):
             return None, None
@@ -616,23 +630,6 @@ class Utilities(script_utilities.Utilities):
     #                                                                       #
     #########################################################################
 
-    def isWordMisspelled(self, obj, offset):
-        """Identifies if the current word is flagged as misspelled by the
-        application.
-
-        Arguments:
-        - obj: An accessible which implements the accessible text interface.
-        - offset: Offset in the accessible's text for which to retrieve the
-          attributes.
-
-        Returns True if the word is flagged as misspelled.
-        """
-
-        attributes, start, end  = self.textAttributes(obj, offset, True)
-        error = attributes.get("text-spelling")
-
-        return error == "misspelled"
-
     def textAttributes(self, acc, offset, get_defaults=False):
         """Get the text attributes run for a given offset in a given accessible
 
